@@ -59,12 +59,14 @@ type DiscoResult struct {
 }
 
 type DiscoX struct {
-	XMLName xml.Name `xml:"x"`
-	Field   []struct {
-		Type  string   `xml:"type,attr"`
-		Var   string   `xml:"var,attr"`
-		Value []string `xml:"value"`
-	} `xml:"field"`
+	XMLName xml.Name      `xml:"x"`
+	Field   []DiscoXField `xml:"field"`
+}
+
+type DiscoXField struct {
+	Type  string   `xml:"type,attr"`
+	Var   string   `xml:"var,attr"`
+	Value []string `xml:"value"`
 }
 
 type DiscoItems struct {
