@@ -2,6 +2,7 @@ package xmpp
 
 import (
 	"fmt"
+	"time"
 )
 
 func (c *Client) PingC2S(jid, server string) error {
@@ -11,10 +12,6 @@ func (c *Client) PingC2S(jid, server string) error {
 	if server == "" {
 		server = c.domain
 	}
-	// Reset ticker for periodic pings if configured.
-	if c.periodicPings {
-		c.periodicPingTicker.Reset(c.periodicPingPeriod)
-	}
 	_, err := fmt.Fprintf(c.stanzaWriter, "<iq from='%s' to='%s' id='%s' type='get'>"+
 		"<ping xmlns='urn:xmpp:ping'/>"+
 		"</iq>\n",
@@ -23,10 +20,6 @@ func (c *Client) PingC2S(jid, server string) error {
 }
 
 func (c *Client) PingS2S(fromServer, toServer string) error {
-	// Reset ticker for periodic pings if configured.
-	if c.periodicPings {
-		c.periodicPingTicker.Reset(c.periodicPingPeriod)
-	}
 	_, err := fmt.Fprintf(c.stanzaWriter, "<iq from='%s' to='%s' id='%s' type='get'>"+
 		"<ping xmlns='urn:xmpp:ping'/>"+
 		"</iq>\n",
@@ -35,10 +28,6 @@ func (c *Client) PingS2S(fromServer, toServer string) error {
 }
 
 func (c *Client) SendResultPing(id, toServer string) error {
-	// Reset ticker for periodic pings if configured.
-	if c.periodicPings {
-		c.periodicPingTicker.Reset(c.periodicPingPeriod)
-	}
 	_, err := fmt.Fprintf(c.stanzaWriter, "<iq type='result' to='%s' id='%s'/>\n",
 		xmlEscape(toServer), xmlEscape(id))
 	return err
@@ -46,6 +35,23 @@ func (c *Client) SendResultPing(id, toServer string) error {
 
 func (c *Client) sendPeriodicPings() {
 	for range c.periodicPingTicker.C {
-		_ = c.PingC2S(c.jid, c.domain)
+		// Reset ticker for periodic pings if configured.
+		if c.periodicPings {
+			c.periodicPingTicker.Reset(c.periodicPingPeriod)
+		}
+		c.periodicPingID = getUUIDv4()
+		c.periodicPingReply = false
+		_, err := fmt.Fprintf(c.stanzaWriter, "<iq from='%s' to='%s' id='%s' type='get'>"+
+			"<ping xmlns='urn:xmpp:ping'/></iq>\n",
+			xmlEscape(c.jid), xmlEscape(c.domain), c.periodicPingID)
+		if err != nil {
+			c.Close()
+		}
+		time.Sleep(c.periodicPingTimeout)
+		if !c.periodicPingReply {
+			c.shutdown = true
+			fmt.Fprintf(c.stanzaWriter, "</stream:stream>\n")
+			c.conn.Close()
+		}
 	}
 }
